@@ -7,7 +7,15 @@
 
 import datetime
 from math import cos, sin, tan, acos, asin, atan2, floor, radians, degrees
+
+SUN_POSITION_CACHE_DURATION = 3600 # seconds
+
 class Astral(object):
+
+    sun_cache_result = None
+    sun_cache_time = None
+    instance = None
+
     def __init__(self):
         """Initialise the list of cities.
         """
@@ -566,11 +574,17 @@ class Astral(object):
 
         return degrees(atan2(tananum, tanadenom))
 
-a = Astral()
-def get_sun_azimuth_from_fix(fix):
-    sunrise = a.sunrise_utc(fix.timestamp, fix.position.lat, -fix.position.lon)
-    sunset = a.sunset_utc(fix.timestamp, fix.position.lat, -fix.position.lon)
-    if fix.timestamp > sunrise and fix.timestamp < sunset:
-        return a.solar_azimuth(fix.timestamp, fix.position.lat, -fix.position.lon)
-    else:
-        return None
+    def get_sun_azimuth_from_fix(self, fix, time = None):
+        if time != None:
+            fix.timestamp = time
+        if self.sun_cache_time == None or abs((fix.timestamp - self.sun_cache_time).seconds) > SUN_POSITION_CACHE_DURATION:
+            self.sun_cache_time = fix.timestamp
+            sunrise = self.sunrise_utc(fix.timestamp, fix.position.lat, -fix.position.lon)
+            sunset = self.sunset_utc(fix.timestamp, fix.position.lat, -fix.position.lon)
+            if fix.timestamp > sunrise and fix.timestamp < sunset:
+                self.sun_cache_result = self.solar_azimuth(fix.timestamp, fix.position.lat, -fix.position.lon)
+            else:
+                self.sun_cache_result = None
+        return self.sun_cache_result
+
+
