@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#   Copyright (C) 2010 Daniel Fett
+#   Copyright (C) 2012 Daniel Fett
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
@@ -25,6 +25,7 @@ import sys
 import geo
 import math
 import os
+import re
 
 usage = r'''Here's how to use this app:
 
@@ -87,7 +88,8 @@ filter-options:
         -o|--owner owner-search-string
         -n|--name name-search-string
         -i|--id id-search-string
-                Search owner, name (title) or id of the geocaches.
+        -a|--attribute attribute-search-string
+                Search owner, name (title), id or attributes of the geocaches (see below for search string syntax).
         --new
                 Caches which were downloaded in current session. Useful to
                 get alerted when new caches arrive.
@@ -121,6 +123,11 @@ Instead of a coordinate, you may also query geonames.com for a place name.
 Just start the string with 'q:':
     q:London
     'q:Brisbane, Australia'
+    
+Search strings are expected as plain strings or as pythonic regular expressions (prefixed with 'r:'). If you use regular expressions, you can use the switch '(?i)' to disable case matching:
+    'title' matches all strings containing title (plain string)
+    r:'.*title.* matches only lower case title (regex)
+    r:'(?i).*title.*' matches also upper case title (regex)
 
 '''
 
@@ -244,17 +251,18 @@ class Cli():
             self.import_points(coord1, coord2)
         elif token == '--around':
             coord1 = self.parse_coord()
-            radius = self.parse_int()
+            radius = self.parse_float()
             self.import_points(coord1, radius)
         elif token == '--at-route':
             coord1 = self.parse_coord()
             coord2 = self.parse_coord()
-            radius = self.parse_int()
+            radius = self.parse_float()
             self.import_points_route(coord1, coord2, radius)
         else:
             # undo what we did.
             self.nt -= 1
             return
+        self.core.pointprovider.save()
             
     def parse_sql(self):
         self.nt += 1
@@ -316,6 +324,9 @@ class Cli():
             elif token == '-i' or token == '--id':
                 id = self.parse_string()
                 self.add_filter_id (id)
+            elif token == '-a' or token == '--attribute':
+                attribute = self.parse_string()
+                self.add_filter_attribute (attribute)
             elif token == '--new':
                 self.caches = self.new_caches
             else:
@@ -414,6 +425,13 @@ class Cli():
         self.nt += 1
         return int(text)
         
+    def parse_float(self):
+        if not self.has_next():
+            raise ParseError("Expected a float, found nothing.", self.nt-1)
+        text = sys.argv[self.nt]
+        self.nt += 1
+        return float(text)
+        
     def parse_size(self):
         if not self.has_next():
             raise ParseError("Expected a size (1..4/micro/small/regular/huge/other), found nothing.", self.nt-1)
@@ -481,13 +499,13 @@ class Cli():
     def import_points(self, c1, c2):
         if isinstance(c2, geo.Coordinate):
             print "* Downloading Caches between %s and %s" % (c1, c2)
-            self.caches, self.new_caches = self.core.on_download((c1, c2))
+            self.caches, self.new_caches = self.core.on_download((c1, c2), sync=True)
         else:
             # try to calculate some points northwest and southeast to the
             # given point with approximately correct distances
             new_c1 = c1.transform(-45, c2 * 1000 * math.sqrt(2))
             new_c2 = c1.transform(-45 + 180, c2 * 1000 * math.sqrt(2))
-            print "* Downloading Caches in %d km distance to %s" % (c2, c1)
+            print "* Downloading Caches in %.3f km distance to %s" % (c2, c1)
             print "* Approximation: Caches between %s and %s" % (new_c1, new_c2)
             self.caches, self.new_caches = self.core.on_download((new_c1, new_c2), sync=True)
 
@@ -564,27 +582,40 @@ class Cli():
         print "* filter with types: %d left" % len(self.caches)
         
     def add_filter_owner(self, owner):
-        self.caches = filter(lambda x: owner.lower() in x.owner.lower(), self.caches)
+        self.caches = filter(lambda x: self.get_string_filter(owner)(x.owner), self.caches)
+        
         print "* filter with owner: %d left" % len(self.caches)
         
     def add_filter_name (self, name):
-        self.caches = filter(lambda x: name.lower() in x.title.lower(), self.caches)
+        self.caches = filter(lambda x: self.get_string_filter(name)(x.title), self.caches)
         print "* filter with name: %d left" % len(self.caches)
     
     def add_filter_id (self, idstring):
-        self.caches = filter(lambda x: idstring.lower() in x.name.lower(), self.caches)
+        self.caches = filter(lambda x: self.get_string_filter(idstring)(x.name), self.caches)
         print "* filter with id: %d left" % len(self.caches)
+        
+    def add_filter_attribute (self, attribute):
+        self.caches = filter(lambda x: self.get_string_filter(attribute)(x.attributes), self.caches)
+        print "* filter with attribute: %d left" % len(self.caches)
+        
+    def get_string_filter(self, searchstring):
+        if searchstring.startswith('r:'):
+            matcher = re.compile(searchstring[2:])
+            return lambda x: matcher.match(x) if x != None else False
+        else:
+            return lambda x: searchstring.lower() in x.lower()
     
     def action_print (self):
         print "Found %d Caches:" % len(self.caches)
         for c in self.caches:
-            print "%s\t%s (%s)" % (c.name, c.title, c.type)
+            print (u"%s\t%s (%s)%s" % (c.name, c.title, c.type, ('*' if c.was_downloaded() else ''))).encode('utf-8')
+            
             
     def action_fetch_details(self):
         i = 1 
         for c in self.caches:
             print "* (%d of %d)\tDownloading '%s'" % (i, len(self.caches), c.title)
-            self.core.on_download_cache(c, sync=True)
+            self.core.on_download_cache_complete(None, self.core.on_download_cache(c, sync=True))
             i += 1
     
     def action_export(self, format, folder):
@@ -595,13 +626,10 @@ class Cli():
             i += 1
 
     def action_command(self, commandline):
-        import unicodedata
         if len(self.caches) == 0:
             print "* Not running command (no geocaches left)"
             return
-        list = " -- ".join(["%s (%s)" % (a.title, a.type) for a in self.caches])
-        if not isinstance(list, str):
-            list = unicodedata.normalize('NFKD', list).encode('ascii','ignore')
+        list = " -- ".join([("%s (%s)" % (a.title, a.type)).encode('utf-8') for a in self.caches])
         os.system(commandline % ('"%s"' % list.encode('string-escape')))
         
     def set_download_progress(self, some, thing):
