@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#   Copyright (C) 2010 Daniel Fett
+#   Copyright (C) 2012 Daniel Fett
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
@@ -26,7 +26,7 @@ TEST = ('''
 
 
 ''', {'A': 2, 'D': 4, 'G': 3,'T': 1, 'R': 2, 'S': 1, 'H': 4, 'B': 2, 'C': 9, 'E': 0, 'F': 1})
-HTML = '''
+HTML = u'''
 
 <br /></p> 
 <p><font face="Arial, sans-serif"><font size=
@@ -74,8 +74,10 @@ class CalcCoordinateManager(object):
         for x in coords:
             logger.debug("Adding: %s, apply_filter = %s" % (x, apply_filter))
             if x.signature in self.__known_signatures:
+                logger.debug("Signature known.")
                 continue
             if apply_filter and x.signature in self.__filtered_signatures:
+                logger.debug("Signature filtered.")
                 continue
             self.__known_signatures.append(x.signature)
             self.requires |= x.requires
@@ -114,6 +116,8 @@ class CalcCoordinateManager(object):
             c.set_vars(self.__vars)
             if c.has_requires():
                 c.try_get_solution()
+            else:
+                c.result = None
 
     def get_solutions(self):
         return [(c.result, c.source) for c in self.coords if c.has_requires() and len(c.requires) > 0]
@@ -168,7 +172,6 @@ class CalcCoordinate():
         return True
 
     def try_get_solution(self):
-
         replaced = [self.__replace(x) for x in [self.lat_deg, self.lat_min, self.lat_min_2, self.lon_deg, self.lon_min, self.lon_min_2]]
         self.replaced_result = ("%%s%s %s.%s %%s%s %s.%s" % tuple(replaced)) % (self.ns, self.ew)
         results = [self.resolve(x) for x in replaced]
@@ -184,6 +187,7 @@ class CalcCoordinate():
             self.result.name = self.orig                
         except (Exception):
             self.warnings.append(self.WARNING_CANNOT_PARSE % result)
+            logger.warning(self.WARNING_CANNOT_PARSE % result)
             self.result = False
         
 
@@ -229,12 +233,11 @@ class CalcCoordinate():
 
     @staticmethod
     def find(text, source):
-        text = re.sub(ur'''(?u)\s[^\W\d_]{2,}\s''', ' | ', text)
-        text = re.sub(ur'''(?u)\b[^\W\d_]{4,}\b''', ' | ', text)
-        text = text.replace('°', '|')
-        text = text.replace(unichr(160), ' ')
-        text = re.sub(ur''' +''', ' ', text)
-        matches = re.findall(ur'''(?<![a-zA-Z])([NSns])\s?([A-Z() -+*/0-9]+?)[\s|]{1,2}%(calc)s[.,\s]%(calc)s['`\s,/]+([EOWeow])\s?([A-Z() -+*/0-9]+?)[\s|]{1,2}%(calc)s[.,\s]%(calc)s[\s'`]*(?![a-zA-Z])''' % {'calc' : CalcCoordinate.SINGLE_CALC_PART}, text)
+        logger.debug("Start parsing...")
+        text = re.sub(ur'(?u)(°|<[^>]+?>|&[a-zA-Z]{,6};| +|%s|\b[^\W\d_]{4,}\b|\s[^\W\d_]{2,}\s)' % unichr(160), ' ', text)
+        logger.debug("End Parsing, matching...")
+        matches = re.findall(ur'''(?<![a-zA-Z])([NSns])\s?([A-Z() -+*/0-9]+?)\s{1,2}%(calc)s[.,\s]%(calc)s['`\s,/]+([EOWeow])\s?([A-Z() -+*/0-9]+?)\s{1,2}%(calc)s[.,\s]%(calc)s[\s'`]*(?![a-zA-Z])''' % {'calc' : CalcCoordinate.SINGLE_CALC_PART}, text)
+        logger.debug("From source %s with text: '''%s''' I found %d matches" % (source, text, len(matches)))
         return [CalcCoordinate(*match, **{'source': source}) for match in matches]
 
     @staticmethod
