@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#   Copyright (C) 2010 Daniel Fett
+#   Copyright (C) 2012 Daniel Fett
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
@@ -22,12 +22,12 @@
 
 from __future__ import with_statement
 
-VERSION = "0.8.0.7"
-   
+VERSION = "0.9.0.0"
 import logging
 logging.basicConfig(level=logging.WARNING,
-                    format='%(relativeCreated)6d %(levelname)10s %(name)-20s %(message)s',
+                    format='%(relativeCreated)6d %(levelname)10s %(name)-20s %(message)s // %(filename)s:%(lineno)s',
                     )
+                    
 
 from geo import Coordinate
 try:
@@ -44,20 +44,20 @@ from os import path, mkdir, extsep, remove, walk
 import provider
 from threading import Thread
 import cachedownloader
-import fieldnotesuploader
 from actors.tts import TTS
 #from actors.notify import Notify
 
 import connection
-
 import gobject
 
+use_connection_handling = False
+
 if len(argv) == 1:
     import cli
     print cli.usage % ({'name': argv[0]})
     exit()
 
-if '-v' in argv:
+if '-v' in argv or '--remote' in argv:
     import colorer
     import logging.handlers
     logging.getLogger('').setLevel(logging.DEBUG)
@@ -69,10 +69,14 @@ if '--simple' in argv:
 elif '--desktop' in argv:
     import biggui
     gui = biggui.BigGui
-elif '--qt' in argv:
-    import qtgui
-    gui = qtgui.QtGui
+elif '--qml-desktop' in argv:
+    import qmlgui_desktop
+    gui = qmlgui_desktop.QmlGui 
+elif '--qml' in argv:
+    import qmlgui
+    gui = qmlgui.QmlGui
 elif '--hildon' in argv:
+    connection.init() # is only used on the maemo platform
     import hildongui
     gui = hildongui.HildonGui
 else:
@@ -107,7 +111,7 @@ class Core(gobject.GObject):
 
     DATA_DIR = path.expanduser(path.join('~', '')) if not path.exists(MAEMO_HOME) else MAEMO_HOME
 
-    UPDATE_MODULES = [cachedownloader, fieldnotesuploader]
+    UPDATE_MODULES = [cachedownloader]
     
     DEFAULT_SETTINGS = {
         'download_visible': True,
@@ -117,7 +121,7 @@ class Core(gobject.GObject):
         'download_create_index': False,
         'download_run_after': False,
         'download_run_after_string': '',
-        'download_output_dir': path.expanduser(path.join(DATA_DIR, 'geocaches', '')),
+        'download_output_dir': path.expanduser(path.join(DATA_DIR, '.geocaches', '')),
         'map_position_lat': 49.7540,
         'map_position_lon': 6.66135,
         'map_follow_position': True,
@@ -137,21 +141,25 @@ class Core(gobject.GObject):
         'options_show_error': True,
         'options_show_html_description': False,
         'map_providers': [
-            ('OpenStreetMaps', {'remote_url': "http://tile.openstreetmap.org/mapnik/%(zoom)d/%(x)d/%(y)d.png", 'prefix': 'OpenStreetMap I'}),
-            ('OpenCycleMaps', {'remote_url': 'http://andy.sandbox.cloudmade.com/tiles/cycle/%(zoom)d/%(x)d/%(y)d.png', 'prefix': 'OpenCycleMap'})
-
+            ('OpenStreetMaps', {'remote_url': "http://a.tile.openstreetmap.org/%(zoom)d/%(x)d/%(y)d.png", 'prefix': 'OpenStreetMap I', 'max_zoom': 18}),
+            ('OpenCycleMaps', {'remote_url': 'http://a.tile.opencyclemap.org/cycle/%(zoom)d/%(x)d/%(y)d.png', 'prefix': 'OpenCycleMap', 'max_zoom': 18}),
         ],
         'options_map_double_size': False,
         'options_rotate_screen': 0,
         'tts_interval': 0,
         'options_default_log_text' : 'TFTC!\n\nLogged at %X from my %(machine)s using AGTL.',
         'options_auto_update': True,
+        'download_num_logs': 20,
+        'options_night_view_mode': 0,
+        'debug_log_to_http': False,
+        'options_backend': 'geocaching-com-new',
     }
             
     def __init__(self, guitype, root):
         gobject.GObject.__init__(self)
         self.current_target = None
         self.current_position = None
+        self.auto_update_checked = False
         self.create_recursive(self.SETTINGS_DIR)
 
         self.dataroot = path.join(root, 'data')
@@ -163,22 +171,26 @@ class Core(gobject.GObject):
         self.connect('save-settings', self.__on_save_settings)
         self.create_recursive(self.settings['download_output_dir'])
         self.create_recursive(self.settings['download_map_path'])
-     
-        connection.init()
 
-        self.downloader = downloader.FileDownloader(self.settings['options_username'], self.settings['options_password'], self.COOKIE_FILE, cachedownloader.GeocachingComCacheDownloader.login_callback)
+        self.downloader = downloader.FileDownloader(self.settings['options_username'], self.settings['options_password'], self.COOKIE_FILE)
                 
         self.pointprovider = provider.PointProvider(self.CACHES_DB, geocaching.GeocacheCoordinate, 'geocaches')
 
         self.gui = guitype(self, self.dataroot)
         
         
-        actor_tts = TTS(self)
-        actor_tts.connect('error', lambda caller, msg: self.emit('error', msg))
+        #actor_tts = TTS(self)
+        #actor_tts.connect('error', lambda caller, msg: self.emit('error', msg))
         #actor_notify = Notify(self)
         self.emit('settings-changed', self.settings, self)
         self.emit('fieldnotes-changed')  
  
+        if ('debug_log_to_http' in self.settings and self.settings['debug_log_to_http']) or '--remote' in argv:
+            http_handler = logging.handlers.HTTPHandler("danielfett.de", "http://www.danielfett.de/files/collect.php")
+            buffering_handler = logging.handlers.MemoryHandler(100, target = http_handler)
+            logging.getLogger('').addHandler(buffering_handler)
+            logging.getLogger('').setLevel(logging.DEBUG)
+            logging.debug("Remote logging activated!")
 
         if '--sim' in argv:
             self.gps_thread = gpsreader.FakeGpsReader(self)
@@ -190,7 +202,10 @@ class Core(gobject.GObject):
             gobject.timeout_add(1000, self.__read_gps)
         elif 'locationgpsprovider' in self.gui.USES:
             self.gps_thread = gpsreader.LocationGpsReader(self.__read_gps_cb_error, self.__read_gps_cb_changed)
-            gobject.idle_add(self.gps_thread.start)  
+            gobject.idle_add(self.gps_thread.start)
+        elif 'qmllocationprovider' in self.gui.USES:
+            self.gui.get_gps(self.__read_gps)
+            self.gps_thread = None
         if 'geonames' in self.gui.USES:
             import geonames
             self.geonames = geonames.Geonames(self.downloader)
@@ -355,7 +370,8 @@ class Core(gobject.GObject):
                         pass
         except Exception, e:
             if sync:
-                raise e
+                self.emit('error', e)
+                return None
             def same_thread(error):
                 logging.exception(e)
                 self.emit('hide-progress')
@@ -398,8 +414,10 @@ class Core(gobject.GObject):
         logger.debug("Settings where changed by %s." % source)
         if source == self:
             return
-        if 'options_username' in settings and 'options_password' in settings:
-            self.downloader.update_userdata(settings['options_username'], settings['options_password'])
+        if 'options_username' in settings:
+            self.downloader.update_userdata(username = settings['options_username'])
+        if 'options_password' in settings:
+            self.downloader.update_userdata(password = settings['options_password'])
 
     '''
     This is called when settings shall be saved, calling save_settings afterwards.
@@ -471,7 +489,7 @@ class Core(gobject.GObject):
         self.current_target = coordinate
         distance, bearing = self.__get_target_distance_bearing()
         self.emit('target-changed', coordinate, distance, bearing)
-        self.emit('map-marks-changed')
+        #self.emit('map-marks-changed')
 
 
     def __get_target_distance_bearing(self):
@@ -483,15 +501,18 @@ class Core(gobject.GObject):
             bearing = None
         return distance, bearing
 
-    def __read_gps(self):
-        fix = self.gps_thread.get_data()
+    def __read_gps(self, fix = None):
+        if fix == None:
+            fix = self.gps_thread.get_data()
 
         if fix.position != None:
             self.current_position = fix.position
             distance, bearing = self.__get_target_distance_bearing()
+            logger.debug("Sending good fix for fix %r" % fix)
             self.emit('good-fix', fix, distance, bearing)
         else:
-            self.emit('no-fix', fix, self.gps_thread.status)
+            logger.debug("Sending bad fix for fix %r" % fix)
+            self.emit('no-fix', fix, self.gps_thread.status if self.gps_thread != None else "")
         return True
 
     def __read_gps_cb_error(self, control, error):
@@ -522,8 +543,6 @@ class Core(gobject.GObject):
     def get_coord_by_name(self, query):
         return self.geonames.search(query)
 
-
-
     def search_place(self, search):
         return self.geonames.search_all(search)
 
@@ -628,12 +647,22 @@ class Core(gobject.GObject):
     #
     ##############################################
 
+    def _check_auto_update(self):
+        if 'options_auto_update' in self.settings and self.settings['options_auto_update'] and not self.auto_update_checked:
+            self.emit('progress', 0.1, "Checking for Updates...")
+            updates = self.try_update(silent = True, sync = True)
+            if updates not in [None, False]:
+                logger.info("Parser update installed.")
+        self.auto_update_checked = True
+
     # called by gui
     def on_download(self, location, sync=False):
+        self._check_auto_update()
         self.emit('progress', 0.5, "Downloading...")
-        cd = cachedownloader.GeocachingComCacheDownloader(self.downloader, self.settings['download_output_dir'], not self.settings['download_noimages'])
+        cd = cachedownloader.get(self.settings['options_backend'], self.downloader, self.settings['download_output_dir'], not self.settings['download_noimages'])
         cd.connect("download-error", self.on_download_error)
         cd.connect("already-downloading-error", self.on_already_downloading_error)
+        cd.connect('progress', self.on_download_progress)
         if not sync:
             def same_thread(arg1, arg2):
                 gobject.idle_add(self.on_download_complete, arg1, arg2)
@@ -682,10 +711,10 @@ class Core(gobject.GObject):
 
     # called by gui
     def on_download_cache(self, cache, sync=False):
-        #
+        self._check_auto_update()
         self.emit('progress', 0.5, "Downloading %s..." % cache.name)
 
-        cd = cachedownloader.GeocachingComCacheDownloader(self.downloader, self.settings['download_output_dir'], not self.settings['download_noimages'])
+        cd = cachedownloader.get(self.settings['options_backend'], self.downloader, self.settings['download_output_dir'], not self.settings['download_noimages'])
         cd.connect("download-error", self.on_download_error)
         cd.connect("already-downloading-error", self.on_already_downloading_error)
         if not sync:
@@ -693,13 +722,13 @@ class Core(gobject.GObject):
                 gobject.idle_add(self.on_download_cache_complete, arg1, arg2)
                 return False
             cd.connect("finished-single", same_thread)
-            t = Thread(target=cd.update_coordinate, args=[cache])
+            t = Thread(target=cd.update_coordinate, args=[cache, self.settings['download_num_logs']])
             t.daemon = True
             t.start()
             #t.join()
             return False
         else:
-            full = cd.update_coordinate(cache)
+            full = cd.update_coordinate(cache, self.settings['download_num_logs'])
             return full
 
     # called on signal by downloading thread
@@ -714,7 +743,7 @@ class Core(gobject.GObject):
 
     # called by gui
     def on_download_descriptions(self, location, visibleonly=False):
-
+        self._check_auto_update()
         #exporter = geocaching.HTMLExporter(self.downloader, self.settings['download_output_dir'])
 
         self.pointprovider.push_filter()
@@ -744,7 +773,8 @@ class Core(gobject.GObject):
 
 
     def update_coordinates(self, caches):
-        cd = cachedownloader.GeocachingComCacheDownloader(self.downloader, self.settings['download_output_dir'], not self.settings['download_noimages'])
+        self._check_auto_update()
+        cd = cachedownloader.get(self.settings['options_backend'], self.downloader, self.settings['download_output_dir'], not self.settings['download_noimages'])
         cd.connect("download-error", self.on_download_error)
         cd.connect("already-downloading-error", self.on_already_downloading_error)
 
@@ -760,13 +790,14 @@ class Core(gobject.GObject):
         cd.connect('progress', self.on_download_progress)
         cd.connect('finished-multiple', same_thread)
 
-        t = Thread(target=cd.update_coordinates, args=[caches])
+        t = Thread(target=cd.update_coordinates, args=[caches, self.settings['download_num_logs']])
         t.daemon = True
         t.start()
 
 
     # called on signal by downloading thread
     def on_download_descriptions_complete(self, something, caches):
+        self._check_auto_update()
         for c in caches:
             self.pointprovider.add_point(c, True)
         self.pointprovider.save()
@@ -779,7 +810,7 @@ class Core(gobject.GObject):
 
     # called on signal by downloading thread
     def on_download_progress(self, something, cache_name, i, max_i):
-        self.emit('progress', float(i) / float(max_i), "Downloading %s (%d of %d)..." % (cache_name, i, max_i))
+        self.emit('progress', float(i) / float(max_i), "Downloading %s (%d of %d)..." % (cache_name, i+1, max_i))
         return False
 
     ##############################################
@@ -803,7 +834,6 @@ class Core(gobject.GObject):
         finally:
             self.emit('hide-progress')
         
-                
 
     ##############################################
     #
@@ -819,15 +849,15 @@ class Core(gobject.GObject):
         elif cache.logas == geocaching.GeocacheCoordinate.LOG_AS_NOTFOUND:
             cache.found = 0
 
-        self.save_cache_attribute(cache, ('logas', 'logdate', 'fieldnotes'))
+        self.save_cache_attribute(cache, ('logas', 'logdate', 'fieldnotes', 'found'))
         self.emit('fieldnotes-changed')        
 
     def on_upload_fieldnotes(self):
         self.emit('progress', 0.5, "Uploading Fieldnotes...")
 
         caches = self.pointprovider.get_new_fieldnotes()
-        fn = fieldnotesuploader.FieldnotesUploader(self.downloader)
-        fn.connect("upload-error", self.on_download_error)
+        fn = cachedownloader.get(self.settings['options_backend'], self.downloader)
+        fn.connect("download-error", self.on_download_error)
         
         def same_thread(arg1):
             gobject.idle_add(self.on_upload_fieldnotes_finished, arg1, caches)
@@ -835,9 +865,7 @@ class Core(gobject.GObject):
             
         fn.connect('finished-uploading', same_thread)
         
-        for c in caches:
-            fn.add_fieldnote(c)
-        t = Thread(target=fn.upload)
+        t = Thread(target=fn.upload_fieldnotes, args=[caches])
         t.daemon = True
         t.start()
         
@@ -845,6 +873,7 @@ class Core(gobject.GObject):
         for c in caches:
             c.logas = geocaching.GeocacheCoordinate.LOG_NO_LOG
             self.save_cache_attribute(c, 'logas')
+            self.emit('cache-changed', c)
         self.emit('hide-progress')
         self.emit('fieldnotes-changed')
 
