@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#   Copyright (C) 2010 Daniel Fett
+#   Copyright (C) 2012 Daniel Fett
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
@@ -23,13 +23,14 @@
 
 import logging
 logger = logging.getLogger('downloader')
+logger.error("Loaded logger")
 import connection
-
+logger.error("Loaded connection")
 class FileDownloader():
-    USER_AGENT = 'User-Agent: Mozilla/5.0 (X11; U; Linux i686; de; rv:1.9.0.12) Gecko/2009070811  Windows NT Firefox/3.1'
+    USER_AGENT = 'User-Agent: Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.15 (KHTML, like Gecko) Ubuntu/11.10 Chromium/18.0.997.0 Chrome/18.0.997.0 Safari/535.15'
     opener_installed = False
 
-    def __init__(self, username, password, cookiefile, login_callback):
+    def __init__(self, username, password, cookiefile):
         self.username = username
         self.password = password
         self.cookiefile = cookiefile
@@ -37,22 +38,27 @@ class FileDownloader():
         from socket import setdefaulttimeout
         setdefaulttimeout(30)
         self.opener_installed = False
-        self.login_callback = login_callback
+        
+        # This controls the use of the cache-headers in requests to allow/deny minified answers
+        # as provided by some mobile operators.
+        self.allow_minified_answers = True
 
-    def update_userdata(self, username, password):
+    def update_userdata(self, username = None, password = None):
         from os import path, remove
-        self.username = username
-        self.password = password
+        if username != None:
+            self.username = username
+        if password != None:
+            self.password = password
         self.logged_in = False
         if path.exists(self.cookiefile):
             try:
                 remove(self.cookiefile)
             except:
-                logger.info("Could not remove cookie file?!")
+                logger.error("Could not remove cookie file?!")
                 pass
 
 
-    def login(self):
+    def login(self, login_callback, check_login_callback):
         if connection.offline:
             raise Exception("Can't connect in offline mode.")
         if self.username == '' or self.password == '':
@@ -74,34 +80,13 @@ class FileDownloader():
             logger.info("Couldn't load cookie file")
         else:
             logger.info("Checking if still logged in...")
-            url = 'http://www.geocaching.com/seek/nearest.aspx'
-            page = self.get_reader(url, login = False)
-            for line in page:
-                if 'Hello, ' in line:
-                    self.logged_in = True
-                    logger.info("Seems as we're still logged in")
-                    page.close()
-                    return
-                elif 'Welcome, Visitor!' in line:
-                    logger.info("Nope, not logged in anymore")
-                    page.close()
-                    break
+            if check_login_callback(self):
+                self.logged_in = True
+                return
         
-        logger.info("Logging in")
-        url, values = self.login_callback(self.username, self.password)
+        logger.info("Logging in with username %s" % self.username)
+        login_callback(self, self.username, self.password)
 
-        page = self.get_reader(url, values, login = False)
-
-        for line in page:
-            if 'You are logged in as' in line:
-                break
-            elif 'Welcome, Visitor!' in line or 'combination does not match' in line:
-                raise Exception("Wrong password or username!")
-        else:
-            logger.info("Seems as if the language is set to something other than english")
-            raise Exception("Please go to geocaching.com and set the website language to english!")
-
-        logger.info("Great success.")
         self.logged_in = True
         try:
             cj.save()
@@ -109,25 +94,33 @@ class FileDownloader():
             logger.info("Could not save cookies: %s" % e)
 
 
-    def get_reader(self, url, values=None, data=None, login = True):
+    def get_reader(self, url, values=None, data=None, login=True, login_callback=None, check_login_callback=None):
+        if login and not (login_callback and check_login_callback):
+            raise Exception("Either login must be set to False or (check_)login_callback must be provided.")
         if connection.offline:
             raise Exception("Can't connect in offline mode.")
         from urllib import urlencode
         from urllib2 import Request, urlopen
         if login and not self.logged_in:
-            self.login()
+            self.login(login_callback, check_login_callback)
 
+        logger.info("Sending request to %s" % url)
+        
+        # No additional parameters or data
         if values == None and data == None:
             req = Request(url)
             self.add_headers(req)
-            return urlopen(req)
+            resp = urlopen(req)
 
+        # There are only URL parameters, expected in values
         elif data == None:
             if (isinstance(values, dict)):
                 values = urlencode( values)
             req = Request(url, values)
             self.add_headers(req)
-            return urlopen(req)
+            resp = urlopen(req)
+            
+        # There are no URL parameters, but a content_type, body tuple in data
         elif values == None:
             content_type, body = data
             req = Request(url)
@@ -135,8 +128,17 @@ class FileDownloader():
             req.add_header('Content-Length', len(str(body)))
             self.add_headers(req)
             req.add_data(body)
-            return urlopen(req)
-
+            resp = urlopen(req)
+        if resp.info().get('Content-Encoding') == 'gzip':
+            from StringIO import StringIO
+            import gzip
+            buf = StringIO(resp.read())
+            logger.debug("Got gzip encoded answer")
+            return gzip.GzipFile(fileobj=buf)
+        else:
+            logger.debug("Got unencoded answer")
+            return resp
+            
     def encode_multipart_formdata(self, fields, files):
         """
         fields is a sequence of (name, value) elements for regular form fields.
@@ -170,5 +172,9 @@ class FileDownloader():
 
     def add_headers(self, req):
         req.add_header('User-Agent', self.USER_AGENT)
-        req.add_header('Cache-Control', 'no-cache')
-        req.add_header('Pragma', 'no-cache')
+        if not self.allow_minified_answers:
+            req.add_header('Cache-Control', 'no-cache')
+            req.add_header('Pragma', 'no-cache')
+        req.add_header('Accept-Encoding', 'gzip')
+        
+logger.error("Loaded all")
