@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#   Copyright (C) 2010 Daniel Fett
+#   Copyright (C) 2012 Daniel Fett
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
@@ -523,17 +523,17 @@ class HildonAboutDialog(object):
         logger.debug("trying update")
         
         import gobject
-        def try_update():
-            if 'options_auto_update' in self.settings and not self.settings['options_auto_update']:
-                return False
-            from time import sleep
-            
-            updates = self.core.try_update(silent = True)
-            
-            if updates not in [None, False]:
-                self.show_success("Parser update installed.")
-            return False
-        gobject.timeout_add(5000, try_update)
+        #def try_update():
+        #    if 'options_auto_update' in self.settings and not self.settings['options_auto_update']:
+        #        return False
+        #    from time import sleep
+        #    
+        #    updates = self.core.try_update(silent = True)
+        #    
+        #    if updates not in [None, False]:
+        #        self.show_success("Parser update installed.")
+        #    return False
+        #gobject.timeout_add(5000, try_update)
 
     def _get_about_button(self):
         button = hildon.Button(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_VERTICAL)
@@ -552,7 +552,7 @@ class HildonAboutDialog(object):
         page = gtk.VBox()
         notebook.append_page(page, gtk.Label('About'))
 
-        copyright = '''Copyright (C) in most parts 2010 Daniel Fett
+        copyright = '''Copyright (C) in most parts 2012 Daniel Fett
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
@@ -827,6 +827,7 @@ class HildonToolsDialog(object):
         
         dialog.show_all()
         result = dialog.run()
+        dialog.hide()
         
     def _show_tool_rot13(self, caller, data = None):
         dialog = gtk.Dialog("ROT13", self.window, gtk.DIALOG_DESTROY_WITH_PARENT, ())
@@ -835,6 +836,7 @@ class HildonToolsDialog(object):
         source = hildon.TextView()
         dialog.vbox.pack_start(source)
         destination = hildon.TextView()
+        destination.set_editable(False)    
         dialog.vbox.pack_start(destination)
         
         def do_rot(widget):
