@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#   Copyright (C) 2010 Daniel Fett
+#   Copyright (C) 2012 Daniel Fett
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
@@ -37,7 +37,7 @@ import geocaching
 import gobject
 import gtk
 import hildon
-from cachedownloader import HTMLManipulations
+from utils import HTMLManipulations
 from hildon_plugins import HildonFieldnotes
 from hildon_plugins import HildonSearchPlace
 from hildon_plugins import HildonSearchGeocaches
@@ -923,7 +923,10 @@ class HildonGui(HildonToolsDialog, HildonSearchPlace, HildonFieldnotes, HildonSe
                 w_name.set_markup(" <b>%s</b>" % my_gtk_label_escape(HTMLManipulations._decode_htmlentities(l['finder'])))
                 w_name.set_alignment(0, 0)
                 w_date = gtk.Label()
-                w_date.set_markup("<b>%4d-%02d-%02d</b>" % (int(l['year']), int(l['month']), int(l['day'])))
+                if 'year' in l:
+                    w_date.set_markup("<b>%4d-%02d-%02d</b>" % (int(l['year']), int(l['month']), int(l['day'])))
+                else:
+                    w_date.set_markup("<b>%s</b>" % l['date'])
                 w_date.set_alignment(0.95, 0)
                 w_text = gtk.Label("%s\n" % l['text'].strip())
                 w_text.set_line_wrap(True)
@@ -955,7 +958,7 @@ class HildonGui(HildonToolsDialog, HildonSearchPlace, HildonFieldnotes, HildonSe
                 widget_hints.pack_start(label_hints, False, False)
             else:
                 label_hints = gtk.Label()
-                label_hints.set_markup('<i>No hints available</i>')
+                label_hints.set_markup('<i>No hints available</i>\nSelect "Download all Details" to get logs, waypoints and images.')
                 widget_hints.pack_start(label_hints, False, False)
 
             # images
@@ -1195,9 +1198,9 @@ class HildonGui(HildonToolsDialog, HildonSearchPlace, HildonFieldnotes, HildonSe
         for x in p.get_children():
             p.remove(x)
         count = len(cache.calc.requires)
-        # create table with n columns.
+        # create table with n rows.
         cols = 7
-        rows = int(ceil(float(count) / float(cols)))
+        rows = max(int(ceil(float(count) / float(cols))), 1)
         table = gtk.Table(rows, cols)
         i = 0
         requires_sort = list(cache.calc.requires)
@@ -1234,6 +1237,7 @@ class HildonGui(HildonToolsDialog, HildonSearchPlace, HildonFieldnotes, HildonSe
 
         # add list vbox
         a = hildon.PannableArea()
+        a.set_property('mov-mode', hildon.MOVEMENT_MODE_BOTH)
         self.cache_calc_vbox = gtk.VBox()
         a.add_with_viewport(self.cache_calc_vbox)
         p.pack_start(a, True)
