@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#   Copyright (C) 2010 Daniel Fett
+#   Copyright (C) 2012 Daniel Fett
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
@@ -49,11 +49,12 @@ except (ImportError):
 import pango
 from os import path, extsep
 import re
-from cachedownloader import HTMLManipulations
+from utils import HTMLManipulations
 from gtkmap import *
 from gui import Gui
 
 
+
 class SimpleGui(Gui):
     USES = ['gpsprovider']
     XMLFILE = "freerunner.glade"
@@ -208,6 +209,7 @@ class SimpleGui(Gui):
         self.check_result_marked = xml.get_widget('check_result_marked')
         self.label_fieldnotes = xml.get_widget('label_fieldnotes')
         self.button_upload_fieldnotes = xml.get_widget('button_upload_fieldnotes')
+        self.button_check_updates = xml.get_widget('button_check_updates')
                 
         self.label_bearing = xml.get_widget('label_bearing')
         self.label_dist = xml.get_widget('label_dist')
@@ -350,12 +352,14 @@ class SimpleGui(Gui):
 
     def _check_notes_save(self):
         if self.current_cache != None and self.notes_changed:
-            self.current_cache.notes = self.cache_elements['notes'].get_text(self.cache_elements['notes'].get_start_iter(), self.cache_elements['notes'].get_end_iter())
+            notes = self.cache_elements['notes'].get_text(self.cache_elements['notes'].get_start_iter(), self.cache_elements['notes'].get_end_iter())
+            self.current_cache.notes = notes.decode('utf-8')
             self.core.save_cache_attribute(self.current_cache, 'notes')
             self.notes_changed = False
 
         if self.current_cache != None and self.fieldnotes_changed:
-            self.current_cache.fieldnotes = self.cache_elements['fieldnotes'].get_text(self.cache_elements['fieldnotes'].get_start_iter(), self.cache_elements['fieldnotes'].get_end_iter())
+            fieldnotes = self.cache_elements['fieldnotes'].get_text(self.cache_elements['fieldnotes'].get_start_iter(), self.cache_elements['fieldnotes'].get_end_iter())
+            self.current_cache.fieldnotes = fieldnotes.decode('utf-8')
             self.core.save_cache_attribute(self.current_cache, 'fieldnotes')
             self.fieldnotes_changed = False
                 
@@ -1042,7 +1046,10 @@ class SimpleGui(Gui):
                     t = 'MAINTENANCE'
                 else:
                     t = l['type'].upper()
-                text_hints += '%s by %s at %4d/%d/%d: %s\n\n' % (t, l['finder'], int(l['year']), int(l['month']), int(l['day']), l['text'])
+                if 'year' in l:
+                    text_hints += '%s by %s at %4d/%d/%d: %s\n\n' % (t, l['finder'], int(l['year']), int(l['month']), int(l['day']), l['text'])
+                else:
+                    text_hints += '%s by %s at %s: %s\n\n' % (t, l['finder'], l['date'], l['text'])
             text_hints += '\n----------------\n'
         else:
             text_hints = 'NO LOGS.\n\n'
@@ -1292,6 +1299,12 @@ class SimpleGui(Gui):
             else:
                 logger.info("Couldn't find widget: input_%s" % x)
         caller.save_settings(settings, self)
+        
+    def on_button_check_updates_clicked(self, caller):
+        updates = self.core.try_update()
+        if updates != None:
+            gobject.idle_add(self.show_success, "%d modules upgraded. There's no need to restart AGTL." % updates)
+        return False
 
 class Updown():
     def __init__(self, table, position, small):
